from pytz import UTC
from unittest.case import TestCase
from datetime import datetime
from icarus.query.update_request_builder import build_add_request
from icarus.test.xml_comparison_util import assert_xml_semantically_equal

__author__ = 'shillaker'


class AddRequestBuilderTest(TestCase):

    def test_add_request_builder(self):
        # Build a document that tests the various encodings
        doc = {
            'basic': 'i\'m a basic string',
            'date': datetime(2014, 8, 1, hour=10, minute=30, second=55, microsecond=100, tzinfo=UTC),
            'boolean': True,
            'number': 100.12345
        }

        # Expected XML result
        expected_xml = '<update><add><doc>'
        expected_xml += '<field name="basic">i\'m a basic string</field>'
        expected_xml += '<field name="date">2014-08-01T10:30:55Z</field>'
        expected_xml += '<field name="boolean">true</field>'
        expected_xml += '<field name="number">100.12345</field>'
        expected_xml += '</doc></add></update>'

        # Expected URL
        expected_url = '/update'

        # Test with commit off
        url, xml = build_add_request(doc, commit=False)
        self.assertEqual(expected_url, url)
        assert_xml_semantically_equal(self, expected_xml, xml)

        # Test with commit on
        expected_url = '/update?commit=true'
        url, xml = build_add_request(doc, commit=True)
        self.assertEqual(expected_url, url)
        assert_xml_semantically_equal(self, expected_xml, xml)