from unittest.case import TestCase
from icarus.query.update_request_builder import build_delete_by_ids_request, build_delete_all_request, \
    build_commit_request

__author__ = 'shillaker'


class DeleteBuilderTest(TestCase):

    # Test creating a commit request
    def test_commit_request(self):
        expected = '/update?commit=true'
        self.assertEqual(build_commit_request(), expected)

    # Test constructing a delete all request
    def test_delete_all_request(self):
        # Test with no commit
        expected_url = '/update'
        expected_xml = '<delete><query>*:*</query></delete>'
        url, xml = build_delete_all_request(False)
        self.assertEqual(url, expected_url)
        self.assertEqual(xml, expected_xml)

        # Test with no commit
        expected_url = '/update?commit=true'
        url, xml = build_delete_all_request(True)
        self.assertEqual(url, expected_url)
        self.assertEqual(xml, expected_xml)

    # Test constructing a delete-by-id request
    def test_delete_by_ids_request(self):
        doc_ids = ['alpha', 'beta', 'delta']

        # Test with no commit
        expected_url = '/update'
        expected_xml = '<update>'
        expected_xml += '<delete><id>alpha</id></delete>'
        expected_xml += '<delete><id>beta</id></delete>'
        expected_xml += '<delete><id>delta</id></delete>'
        expected_xml += '</update>'

        expected_xml = expected_xml.strip(' \t\n\r')

        url, xml = build_delete_by_ids_request(doc_ids, False)
        self.assertEqual(url, expected_url)
        self.assertEqual(xml, expected_xml)

        # Test with no commit
        expected_url = '/update?commit=true'
        url, xml = build_delete_by_ids_request(doc_ids, True)
        self.assertEqual(url, expected_url)
        self.assertEqual(xml, expected_xml)