from unittest.case import TestCase

from mock import Mock
from icarus.query.query_classes import SearchQuery, AND, OR
from icarus.test.query_comparison_util import check_query_url


__author__ = 'shillaker'


# Tests for the select query builder
class SelectBuilderTest(TestCase):

    def setUp(self):
        self.interface = Mock()
        self.query = SearchQuery(self.interface)

    # Checks adding filter queries
    def test_fq(self):
        self.query.fq(AND, 'blah', 'alpha')
        expected = '/select?q=*&fq=blah:"alpha"'
        check_query_url(self, self.query, expected)

        self.query.fq(OR, 'blah2', [12, 14], AND)
        expected += '+OR+blah2:(12+AND+14)'
        check_query_url(self, self.query, expected)

        self.query.fq(OR, 'blah3', False)
        expected += '+OR+blah3:false'
        check_query_url(self, self.query, expected)

    # Checks adding filter queries to existing queries
    def test_fq_with_q(self):
        self.query.q(AND, 'blah', 'alpha')
        expected = '/select?q=blah:"alpha"'
        check_query_url(self, self.query, expected)

        self.query.fq(OR, 'blah2', [12, 14], AND)
        expected += '&fq=blah2:(12+AND+14)'
        check_query_url(self, self.query, expected)

        self.query.fq(OR, 'blah3', False)
        expected += '+OR+blah3:false'
        check_query_url(self, self.query, expected)
