from unittest.case import TestCase

from mock import Mock
from icarus.query.query_classes import SearchQuery, OR, DESCENDING
from icarus.query.select_request_builder import is_single_value
from icarus.test.query_comparison_util import check_query_url


__author__ = 'shillaker'


# Tests for the select query builder
class SelectBuilderTest(TestCase):

    def setUp(self):
        self.interface = Mock()
        self.query = SearchQuery(self.interface)

    # Checks we're classifying things as single-valued properly
    def test_single_value(self):
        self.assertTrue(is_single_value('blahblah'), 'String should be a single value')
        self.assertTrue(is_single_value(None), 'None should be a single value')
        self.assertTrue(is_single_value(True), 'Boolean should be a single value')
        self.assertTrue(is_single_value(12345.2), 'Float should be a single value')
        self.assertTrue(is_single_value(123), 'Integer should be a single value')

        self.assertFalse(is_single_value([1, 2, 3, 4]), 'List should not be a single value')
        self.assertFalse(is_single_value((1, 2, 3, 4)), 'Tuple should not be a single value')
        self.assertFalse(is_single_value({1, 2, 3, 4}), 'Set should not be a single value')

    # Tests adding a wildcard
    def test_star_wildcard(self):
        self.query.q(OR, 'blah', '*')
        check_query_url(self, self.query, '/select?q=blah:*')

    # Tests adding a sort field
    def test_sort(self):
        self.query.q(OR, 'blah', 'alpha')
        self.query.sort('blah2')
        check_query_url(self, self.query, '/select?q=blah:"alpha"&sort=blah2+asc')

    # Tests adding a sort field
    def test_sort_order(self):
        self.query.q(OR, 'blah', 'alpha')
        self.query.sort('blah2', DESCENDING)
        check_query_url(self, self.query, '/select?q=blah:"alpha"&sort=blah2+desc')

    # Tests adding a result fields
    def test_result_fields(self):
        self.query.q(OR, 'blah', 'alpha')
        self.query.result_fields(['alpha', 'beta', 'gamma'])
        check_query_url(self, self.query, '/select?q=blah:"alpha"&fl=alpha,beta,gamma')

    # Tests buiding a request with facets
    def test_facet_request(self):
        self.query.q(OR, 'blah', 'alpha')
        self.query.facet_fields(['blah2', 'blah3'])
        check_query_url(self, self.query, '/select?q=blah:"alpha"&facet.field=blah2&facet.field=blah3&facet=true')

