from unittest.case import TestCase

from mock import Mock
from icarus.query.query_classes import SearchQuery, OR, AND
from icarus.test.query_comparison_util import check_query_url


__author__ = 'shillaker'


# Tests for the select query builder
class SelectBuilderTest(TestCase):

    def setUp(self):
        self.interface = Mock()
        self.query = SearchQuery(self.interface)

    # Checks adding single valued fields
    def test_q(self):
        # First subquery
        self.query.q(OR, 'blah', True)
        expected = '/select?q=blah:true'
        check_query_url(self, self.query, expected)
        # Second subquery
        self.query.q(AND, 'blah2', '*')
        expected += '+AND+blah2:*'
        check_query_url(self, self.query, expected)
        # Third subquery
        self.query.q(OR, 'blah3', ['gamma gamma', 'delta delta'], AND)
        expected += '+OR+blah3:("gamma+gamma"+AND+"delta+delta")'
        check_query_url(self, self.query, expected)

    # Checks adding multiple values at once
    def test_multi_q(self):
        # Add multiple Q values at once
        self.query.q_multi(AND, {
            'blah': ['alpha', 'beta'],
            'blah2': [12.3, 10.5],
            'blah3': 'epsilon'
        }, OR)
        expected = '/select?q=blah:("alpha"+OR+"beta")+AND+blah2:(12.3+OR+10.5)+AND+blah3:"epsilon"'
        check_query_url(self, self.query, expected)

        # Add another couple
        self.query.q_multi(OR, {
            'blah4': True,
            'blah5': 100,
        })
        expected += '+OR+blah4:true+OR+blah5:100'
        check_query_url(self, self.query, expected)