from unittest.case import TestCase
from icarus.result.select_response_handler import build_select_result


__author__ = 'shillaker'


class SelectResponseTest(TestCase):
    # Test that responses to select requests are dealt with properly
    def test_select_response_handling(self):
        n_total = 10

        docs = [
            {
                'blah': False,
                'blah2': 'alpha'
            },
            {
                'blah': True,
                'blah2': 'beta'
            }
        ]

        dummy_response = {
            'responseHeader': {
                'status': 0,
                'QTime': 2
            },
            'response': {
                'numFound': n_total,
                'start': 0,
                'docs': docs
            },
            'facet_counts': {
                'facet_queries': {},
                'facet_fields': {
                    'source': ['alpha', 10, 'beta', 50]
                },
                'facet_dates': {},
                'facet_ranges': {}
            }
        }

        expected_facets = {
            'source': [
                ('alpha', 10),
                ('beta', 50)
            ]
        }

        actual = build_select_result(dummy_response)

        # Check facets are equal
        self.assertDictEqual(expected_facets, actual.facet_counts)

        # Check all docs are equal
        for i in range(0, len(docs)):
            self.assertDictEqual(docs[i], actual.items[i])

        # Check number of results reported correctly
        self.assertEqual(n_total, actual.n_total)